import requests


class OAuth2:
    """Authorization object using OAuth2 to define API info."""

    def __init__(self, base_url: str, client_id: str, client_secret: str, scope: str):
        """
        A constructor function. It is called when an object of the class is created.
        
        :param base_url: https://api.box.com/2.0
        :type base_url: str
        :param client_id: The client ID of the application you registered in Azure AD
        :type client_id: str
        :param client_secret: The client secret you received from the developer portal
        :type client_secret: str
        :param scope: The scope of the access request expressed as a list of space-delimited,
        case-sensitive strings. The strings are defined by the host server
        :type scope: str
        """
        self.base_url = base_url
        self.auth_token = self._authenticate(client_id, client_secret, scope)

    def _authenticate(self, client_id: str, client_secret: str, scope: str) -> str:
        """
        It takes in a client_id, client_secret, and scope, and returns an access token.
        
        :param client_id: The client ID of the application you created in the developer portal
        :type client_id: str
        :param client_secret: The client secret for the application
        :type client_secret: str
        :param scope: 'https://api.ebay.com/oauth/api_scope'
        :type scope: str
        :return: The access token
        """

        url = self.base_url + '/identity/connect/token'

        headers = {
            'Content-Type': 'application/x-www-form-urlencoded'
        }

        payload = {
            'grant_type': 'client_credentials',
            'client_id': client_id,
            'client_secret': client_secret,
            'scope': scope
        }

        r = requests.post(url=url, headers=headers, data=payload)

        if r.status_code == 200:
            print('Authenticated Successfully')
            return_value = r.json()
            auth = f"{return_value['token_type']} {return_value['access_token']}"
            return auth

        else:
            raise ValueError(
                "Server Error: " + str(r.status_code) +
                '.  ' + r.json()['error']
            )


from .OAuth2 import OAuth2
from typing import Tuple, Any

import requests
import json


class QueueDefinitions:

    def __init__(self, auth: OAuth2, folder_id: str):
        """
        This function is used to initialize the class
        
        :param auth: OAuth2
        :type auth: OAuth2
        :param folder_id: The ID of the folder you want to upload the file to
        :type folder_id: str
        """

        self.auth = auth
        self.folder_id = folder_id

    def getQueueDefinitions(self):
        """
        It returns a list of all the queues in the Orchestrator instance
        :return: A list of queue definitions.
        """

        url = self.auth.base_url + '/odata/QueueDefinitions'

        payload = {}

        headers = {
            'X-UIPATH-OrganizationUnitId': self.folder_id,
            'Authorization': self.auth.auth_token
        }

        r = requests.get(url=url, headers=headers, data=payload)

        if r.status_code == 200:
            return_value = r.json()
            return return_value

        else:
            raise ValueError(
                "Server Error: " + str(r.status_code) +
                ".  " + r.json()['message']
            )

    def checkQueueDefinitionExists(self, queue_name: str) -> Tuple[bool, Any]:
        """
        It checks if a queue exists in a folder
        
        :param queue_name: The name of the queue you want to check for
        :type queue_name: str
        :return: A boolean value.
        """

        queueExists: bool = None

        existingQueueDefinitions = self.getQueueDefinitions()
        queue_def = existingQueueDefinitions

        for queue in existingQueueDefinitions['value']:
            existingQueueName = str(queue['Name'])
            if existingQueueName.strip() == queue_name.strip():
                queueExists = True
                queue_def = queue
                break
            else:
                queueExists = False

        return (queueExists, queue_def)

    def createNewQueueDefinition(self, queue_name: str, description: str, max_retries: int, auto_retry=False, enforce_unique=False) -> Any:
        """
        This function creates a new queue definition in Orchestrator
        
        :param queue_name: The name of the queue you want to create
        :type queue_name: str
        :param description: str,
        :type description: str
        :param max_retries: The maximum number of times a job can be retried
        :type max_retries: int
        :param auto_retry: If true, the queue item will be automatically retried if the robot fails to
        process it, defaults to False (optional)
        :param enforce_unique: If true, the queue item will be rejected if the reference is not unique,
        defaults to False (optional)
        :return: A tuple containing a boolean and a dictionary.
        """

        url = self.auth.base_url + '/odata/QueueDefinitions'

        payload = json.dumps({
            "Name": queue_name,
            "Description": description,
            "MaxNumberOfRetries": max_retries,
            "AcceptAutomaticallyRetry": auto_retry,
            "EnforceUniqueReference": enforce_unique
        })

        headers = {
            'Content-Type': 'application/json',
            'X-UIPATH-OrganizationUnitId': self.folder_id,
            'Authorization': self.auth.auth_token
        }

        (qDefExists, qDef) = self.checkQueueDefinitionExists(queue_name)

        if qDefExists:
            print(f'Queue {queue_name} already exists and cannot be created.')
            return qDef
        else:
            r = requests.post(url=url, headers=headers, data=payload)

            if r.status_code == 201:
                print(f'Queue {queue_name} created.')
                return_value = r.json()
                return return_value

            else:
                raise ValueError(
                    "Server Error: " + str(r.status_code) +
                    ".  " + r.json()['message']
                )


from .OAuth2 import OAuth2

import requests
import json


class QueueItems:

    def __init__(self, auth: OAuth2, folder_id: str):
        """
        This function is used to initialize the class
        
        :param auth: OAuth2
        :type auth: OAuth2
        :param folder_id: The ID of the folder you want to upload the file to
        :type folder_id: str
        """

        self.auth = auth
        self.folder_id = folder_id

    def addQueueItem(self, queue_name: str, queue_item_reference: str, specific_content: dict, priority='Normal') -> int:
        """
        This function will add a queue item to a queue in Orchestrator
        
        :param queue_name: The name of the queue you want to add the item to
        :type queue_name: str
        :param queue_item_reference: This is the name of the queue item
        :type queue_item_reference: str
        :param specific_content: dict
        :type specific_content: dict
        :param priority: 'High', 'Normal', 'Low', defaults to Normal (optional)
        :return: The ID of the queue item that was created.
        """

        url = self.auth.base_url + '/odata/Queues/UiPathODataSvc.AddQueueItem'

        payload = json.dumps({
            "itemData": {
                "Name": queue_name,
                "Priority": priority,
                "Reference": queue_item_reference,
                "SpecificContent": specific_content
            }
        })

        headers = {
            'Content-Type': 'application/json',
            'X-UIPATH-OrganizationUnitId': self.folder_id,
            'Authorization': self.auth.auth_token
        }

        r = requests.post(url=url, headers=headers, data=payload)

        if r.status_code == 201:
            print(f"Queue Item '{queue_item_reference}' created.")
            return r.json()['Id']

        else:
            raise ValueError(
                "Server Error: " + str(r.status_code) +
                ".  " + r.json()['message']
            )

    def getQueueItem(self, queue_item_id: int):
        """
        This function will retrieve a queue item from Orchestrator based on the queue item ID.
        
        :param queue_item_id: The ID of the queue item you want to retrieve
        :type queue_item_id: int
        :return: A JSON object containing the queue item data.
        """

        url = self.auth.base_url + f'/odata/QueueItems({str(queue_item_id)})'

        payload = {}

        headers = {
            'Content-Type': 'application/json',
            'X-UIPATH-OrganizationUnitId': self.folder_id,
            'Authorization': self.auth.auth_token
        }

        r = requests.get(url=url, headers=headers, data=payload)

        if r.status_code == 200:
            data = r.json()
            print(f"Queue Item '{data['Reference']}' retrieved.")
            return data

        else:
            raise ValueError(
                "Server Error: " + str(r.status_code) +
                ".  " + r.json()['message']
            )
